(* Delphi Dialog
   Besttigungsdialog mit CheckBox
   ===============================
   
    J. Rathlev, D.24222 Schwentinental (info(a)rathlev-home.de))

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Feb. 2014
   *)

unit ConfirmCheckDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Imaging.pngimage;

type
  TConfirmCheckDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    laMsg: TLabel;
    cbConfirm: TCheckBox;
    imgQMark: TImage;
    procedure FormCreate(Sender: TObject);
    procedure cbConfirmClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function Execute (APos : TPoint; const ATitle,Desc,Confirm : string) : TModalresult;
  end;

function ConfirmCheck (APos : TPoint; const Desc,Confirm : string) : TModalresult;

var
  ConfirmCheckDialog: TConfirmCheckDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses WinUtils, GnuGetText;

var
  IniFileName,SectionName   : string;

{ ---------------------------------------------------------------- }
procedure TConfirmCheckDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{ ---------------------------------------------------------------- }
procedure TConfirmCheckDialog.cbConfirmClick(Sender: TObject);
begin
  with OKBtn do if cbConfirm.Checked then begin
    ModalResult:=mrYes;
    Caption:=dgettext('dialogs','Yes');
    end
  else begin
    ModalResult:=mrNo;
    Caption:=dgettext('dialogs','No');
    end;
  end;

function TConfirmCheckDialog.Execute (APos : TPoint; const ATitle,Desc,Confirm : string) : TModalresult;
begin
  with APos do begin
    if (Y < 0) or (X < 0) then Position:=poScreenCenter
    else begin
      if X<0 then X:=Left;
      if Y<0 then Y:=Top;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    end;
  if length(ATitle)>0 then Caption:=ATitle;
  laMsg.Caption:=Desc;
  with cbConfirm do begin
    Caption:=Confirm;
    Checked:=false;
    end;
  Result:=ShowModal;
  end;

function ConfirmCheck(APos : TPoint; const Desc,Confirm : string) : TModalresult;
begin
  if not assigned(ConfirmCheckDialog) then
    ConfirmCheckDialog:=TConfirmCheckDialog.Create(Application);
  Result:=ConfirmCheckDialog.Execute(APos,'',Desc,Confirm);
  FreeAndNil(ConfirmCheckDialog);
  end;

end.
