(* Delphi dialog
   Edit password with confirmation

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - June 2005
   *)
   
unit AccountDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

const
  defMinPwdLength = 6;

type
  TAccountDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    edtRepPwd: TEdit;
    Label1: TLabel;
    edtPwd: TEdit;
    btnShow: TSpeedButton;
    Label2: TLabel;
    edtName: TEdit;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FConf   : boolean;
    FMinLen : integer;
  public
    { Public declarations }
    function Execute (Title : string; var Username,Pwd : string;
                      MinLen   : integer; Confirm,Show : boolean) : boolean;
  end;

function InputAccount (Title : string; var Username,Pwd : string;
                      MinLen   : integer; Confirm,Show : boolean) : boolean;

var
  AccountDialog: TAccountDialog;

implementation

{$R *.DFM}

uses WinUtils;

{------------------------------------------------------------------- }
procedure TAccountDialog.FormCreate(Sender: TObject);
begin
  // ***
  end;

procedure TAccountDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) and FConf then
    CanClose:=(edtPwd.Text=edtRepPwd.Text) and (length(edtPwd.Text)>=FMinLen);
  if not CanClose then begin
    if length(edtPwd.Text)<FMinLen then begin
      ErrorDialog(Caption,Format('Das Passwort muss mindestens %u Zeichen lang sein!',[FMinLen]),CenterPos);
      edtPwd.SetFocus;
      end
    else begin
      ErrorDialog(Caption,'Die Passwortangaben stimmen nicht berein!),CenterPos);
      edtRepPwd.SetFocus;
      end;
    end;
  end;

procedure TAccountDialog.btnShowClick(Sender: TObject);
var
  c : char;
begin
  if btnShow.Down then c:=#0 else c:='*';
  edtPwd.PasswordChar:=c;
  edtRepPwd.PasswordChar:=c;
  end;

{ ------------------------------------------------------------------- }
(* Passworteingabe, Ergebnis: "true" bei "ok" *)
function TAccountDialog.Execute (Title    : string;
                                var Username,Pwd  : string;
                                MinLen   : integer;
                                Confirm,Show  : boolean) : boolean;
begin
  Caption:=Title;
  edtName.Text:=Username;
  edtPwd.Text:=Pwd;
  edtRepPwd.Text:=Pwd;
  FConf:=Confirm;
  FMinLen:=MinLen;
  btnShow.Visible:=Show;
  if FConf then edtRepPwd.Show
  else edtRepPwd.Hide;
  ActiveControl:=edtName;
  if ShowModal=mrOK then begin
    Username:=edtName.Text;
    Pwd:=edtPwd.Text;
    Result:=true;
    end
  else Result:=false;
  end;

{------------------------------------------------------------------- }
function InputAccount (Title : string; var Username,Pwd : string;
                      MinLen   : integer; Confirm,Show : boolean) : boolean;
begin
  if not assigned(AccountDialog) then AccountDialog:=TAccountDialog.create(Application);
  Result:=AccountDialog.execute(Title,Username,Pwd,MinLen,Confirm,Show);
  FreeAndNil(AccountDialog);
  end;

end.
