(* Delphi Unit
   Edit titles and widths of TListView columns

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Dec. 2008
   *)

unit ColumnDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls;

type
  TColumnDialog = class(TForm)
    btbOK: TBitBtn;
    btbCancel: TBitBtn;
    lvColumns: TListView;
    edWidth: TLabeledEdit;
    udWidth: TUpDown;
    btnInsert: TBitBtn;
    btnDelete: TBitBtn;
    btnEdit: TBitBtn;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    Label1: TLabel;
    cbFields: TComboBox;
    rgSort: TRadioGroup;
    procedure UpBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure lvColumnsClick(Sender: TObject);
    procedure btnInsertClick(Sender: TObject);
    procedure cbFieldsCloseUp(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FixColumns: TStringList;
  public
    { Public declarations }
    function Execute (APos : TPoint; ExtraFields : array of string; const AFixColumns : array of string;
                      FieldList : TStringList; ColList : TListColumns) : boolean;
  end;

var
  ColumnDialog: TColumnDialog;

implementation

{$R *.dfm}

uses WinUtils, StringUtils;

{ ------------------------------------------------------------------- }
procedure TColumnDialog.FormCreate(Sender: TObject);
begin
  FixColumns:=TStringList.Create;
  end;

procedure TColumnDialog.FormDestroy(Sender: TObject);
begin
  FixColumns.Free;
  end;

{ ------------------------------------------------------------------- }
procedure TColumnDialog.UpBtnClick(Sender: TObject);
var
  n  : integer;
  li : TListItem;
begin
  with lvColumns,Items do if (Count>0) and (ItemIndex>0) then begin
    n:=ItemIndex;
    li:=Insert(n-1);
    li.Assign(Item[n+1]);
    Delete(n+1);
    ItemIndex:=n-1;
    end;
  end;

procedure TColumnDialog.DownBtnClick(Sender: TObject);
var
  n  : integer;
  li : TListItem;
begin
  with lvColumns,Items do if (Count>0) and (ItemIndex<Count-1) then begin
    n:=ItemIndex;
    li:=Insert(n+2);
    li.Assign(Item[n]);
    Delete(n);
    ItemIndex:=n+1;
    end;
  end;

procedure TColumnDialog.cbFieldsCloseUp(Sender: TObject);
var
  n : integer;
begin
  n:=FixColumns.IndexOf(cbFields.Text);
  with udWidth do if n>=0 then begin
    Position:=20; Enabled:=false;
    end
  else begin
    Position:=50; Enabled:=true;
    end;
  edWidth.Enabled:=udWidth.Enabled;
  end;

procedure TColumnDialog.btnInsertClick(Sender: TObject);
begin
  with lvColumns.Items.Add do begin
    Caption:=cbFields.Text;
    Data:=pointer(udWidth.Position+rgSort.ItemIndex shl 12);
    SubItems.Add(edWidth.Text);
    if rgSort.ItemIndex=0 then SubItems.Add('T') else SubItems.Add('N');
    end;
  end;

procedure TColumnDialog.btnEditClick(Sender: TObject);
begin
  with lvColumns do if ItemIndex>=0 then with Selected do begin
    Caption:=cbFields.Text;
    Data:=pointer(udWidth.Position+rgSort.ItemIndex shl 12);
    SubItems[0]:=edWidth.Text;
    if rgSort.ItemIndex=0 then SubItems[1]:='T' else SubItems[1]:='N';
    end;
  end;

procedure TColumnDialog.btnDeleteClick(Sender: TObject);
begin
  with lvColumns do if ItemIndex>=0 then Items.Delete(ItemIndex);
  end;

procedure TColumnDialog.lvColumnsClick(Sender: TObject);
begin
  with lvColumns do if assigned(Selected) then with Selected do begin
    with cbFields,Items do ItemIndex:=IndexOf(Caption);
    with udWidth do begin
      Position:=integer(Data) and $FFF;
      Enabled:=FixColumns.IndexOf(Caption)<0;
      edWidth.Enabled:=Enabled;
      end;
    rgSort.ItemIndex:=(integer(Data) and $1000) shr 12;
    end;
  end;

function TColumnDialog.Execute (APos : TPoint; ExtraFields : array of string; const AFixColumns : array of string;
                                FieldList : TStringList; ColList : TListColumns) : boolean;
var
  i,n : integer;
  s,t : string;
begin
  with APos do begin
    if (Y < 0) or (X < 0) then Position:=poScreenCenter
    else begin
      Position:=poDesigned;
      if X<0 then X:=Left;
      if Y<0 then Y:=Top;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    end;
  FixColumns.Clear;
  for i:=0 to High(AFixColumns) do begin
    s:=AFixColumns[i];
    t:=ReadNxtStr(s,VertBar);
    n:=ReadNxtInt(s,VertBar,20);
    FixColumns.AddObject(t,pointer(n));
    end;
  with cbFields do begin
    Clear;
    if length(ExtraFields)>0 then
      for i:=Low(ExtraFields) to High(ExtraFields) do Items.Add(ExtraFields[i]);
    for i:=0 to FieldList.Count-1 do Items.Add(FieldList[i]);
    end;
  lvColumns.Clear;
  for i:=1 to ColList.Count-1 do begin
    with ColList.Items[i] do begin
      s:=Caption;
      n:=Width;
      if Tag=0 then t:='T'
      else begin
        t:='N';
        n:=n or $1000;
        end;
      end;
    with lvColumns.Items.Add do begin
      Caption:=s;
      Data:=pointer(n);
      SubItems.Add(IntToStr(n and $FFF));
      SubItems.Add(t);
      end
    end;
  if ShowModal=mrOK then begin
    with ColList do for i:=Count-1 downto 1 do Delete(i);
    with lvColumns.Items do for i:=0 to Count-1 do with Item[i] do begin
      s:=Caption;
      with ColList.Add do begin
        Caption:=s;
        Width:=integer(Data) and $FFF;
        Tag:=(integer(Data) and $1000) shr 12;
        end;
      end;
    Result:=true;
    end
  else Result:=false;
  end;
  
end.
