(* Delphi Dialog
   Dialog: Eingabe der FTP-Verbindungsparameter
   ============================================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Dez. 2006
   Vers. 2 - Jul. 2007
   Vers. 3 - Aug. 2008   
   *)

unit FtpDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls,
  NumberEd, HListBox, FtpFileTools, IdAssignedNumbers;

const
  defFtpPort = IdPORT_FTP;
  defProxyPort = 2121;

type
  TFtpDialog = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    hcbServer: THistoryCombo;
    hcbUser: THistoryCombo;
    hcbDir: THistoryCombo;
    edtFtpPwd: TEdit;
    Label5: TLabel;
    edtPort: TRangeEdit;
    btnShow: TSpeedButton;
    cbxCase: TComboBox;
    Label6: TLabel;
    cbProxy: TCheckBox;
    edProxy: TEdit;
    btProxy: TBitBtn;
    pnSettings: TPanel;
    pnTitle: TPanel;
    edName: TLabeledEdit;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    gbSecure: TGroupBox;
    rbNone: TRadioButton;
    rbAvail: TRadioButton;
    rbAlways: TRadioButton;
    rbRequired: TRadioButton;
    gbOptions: TGroupBox;
    cbPassive: TCheckBox;
    cbUseHost: TCheckBox;
    cbUtf8: TCheckBox;
    cbFtpLog: TCheckBox;
    cbQuotes: TCheckBox;
    procedure btnShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbProxyClick(Sender: TObject);
    procedure btProxyClick(Sender: TObject);
    procedure rbNoneClick(Sender: TObject);
    procedure rbAvailClick(Sender: TObject);
    procedure rbRequiredClick(Sender: TObject);
    procedure rbAlwaysClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private-Deklarationen }
    FIniName : string;
    sp    : boolean;
    FSecureMode : integer;
    FProxyPar   : TFtpProxyParams;
    procedure SetPassiv;
    procedure ShowProxy;
  public
    { Public-Deklarationen }
    procedure Init (AIniName,DefIni : string);
    function DefaultValues : TFtpParams;
    function Execute (var AFtpPar : TFtpParams; ShowPwd,ShowSecure : boolean;
                      ShowTitle : boolean = false) : boolean;
  end;

function GetFtpHint (FtpPar : TFtpParams) : string;

function GetFtpSecMode (SecMode : integer) : string;

var
  FtpDialog: TFtpDialog;

implementation

{$R *.dfm}

uses IniFiles, IdFtp, StringUtils, WinUtils, FtpProxyDlg, Web.HTTPApp;

{ ------------------------------------------------------------------- }
function GetFtpHint(FtpPar : TFtpParams) : string;
begin
  with FtpPar do begin
    if SecureMode>0 then Result:='ftps' else Result:='ftp';
    Result:=Result+'://'+Host+Colon+IntToStr(Port)+'/'+Directory;
    end;
  end;

function GetFtpSecMode (SecMode : integer) : string;
begin
  case SecMode of
  1 : Result:=FtpDialog.rbAvail.Caption;
  2 : Result:=FtpDialog.rbRequired.Caption;
  3 : Result:=FtpDialog.rbAlways.Caption;
  else Result:=FtpDialog.rbNone.Caption;
    end;
  Result:='Sichere bertragung: '+Result;
  end;

{ ------------------------------------------------------------------- }
procedure TFtpDialog.FormCreate(Sender: TObject);
begin
  FIniName:=''; sp:=true;
  end;

const
  ServerSekt = 'FtpServer';
  UserSekt   = 'Usernames';
  DirSekt    = 'FtpDirs';

procedure TFtpDialog.Init (AIniName,DefIni : string);
var
  si      : string;
begin
  // nur beim Neustart laden
  if (length(FIniName)=0) and ((length(AIniName)>0) or (length(DefIni)>0)) then begin
    FIniName:=AIniName;
    if FileExists(FIniName) then si:=FIniName else si:=DefIni;
    if FileExists(si) then begin
      hcbServer.LoadFromIni(si,ServerSekt);
      hcbUser.LoadFromIni(si,UserSekt);
      hcbDir.LoadFromIni(si,DirSekt);
      end;
    end;
  end;

function TFtpDialog.DefaultValues : TFtpParams;
begin
  with Result do begin
    Host:='';
    Port:=defFtpPort;
    Username:='';
    Password:='';
    Directory:='';
    SecureMode:=0;
    ConnTimeout:=defTimeOut;
    ConnAttempts:=defConnect;
    Passive:=true;
    UseHost:=false;
    WriteLog:=false;
    ForceUtf8:=false;
    CaseMode:=tcNone;
    Quotes:=false;
    InvalidChars:='';
    with Proxy do begin
      Server:='';
      Username:='';
      Password:='';
      Port:=defProxyPort;
      Mode:=fpcmNone;
      end;
    end;
  end;

procedure TFtpDialog.btnShowClick(Sender: TObject);
begin
  with edtFtpPwd do if btnShow.Down then PasswordChar:=#0 else PasswordChar:='*';
  end;

procedure TFtpDialog.ShowProxy;
begin
  with FProxyPar do begin
    if cbProxy.Checked and (length(Server)>0) then edProxy.Text:=Server+':'+IntToStr(Port)
    else edProxy.Text:='';
    edProxy.Enabled:=cbProxy.Checked;
    btProxy.Enabled:=cbProxy.Checked;
    end;
  end;

procedure TFtpDialog.cbProxyClick(Sender: TObject);
begin
  if Visible and cbProxy.Checked then begin
    EditFtpProxy(FProxyPar,sp);
    cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
    end;
  ShowProxy;
  end;

procedure TFtpDialog.btProxyClick(Sender: TObject);
begin
    FtpProxyDialog.Execute (FProxyPar,sp);
    cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
  ShowProxy;
  end;

procedure TFtpDialog.SetPassiv;
begin
  with cbPassive do begin
    Enabled:=fSecureMode=0;
    Checked:=true;            // muss auf "passiv" stehen
    end;
  end;

procedure TFtpDialog.rbNoneClick(Sender: TObject);
begin
  fSecureMode:=0;
  end;

procedure TFtpDialog.rbRequiredClick(Sender: TObject);
begin
  fSecureMode:=2;
  end;

procedure TFtpDialog.rbAvailClick(Sender: TObject);
begin
  fSecureMode:=1;
  end;

procedure TFtpDialog.rbAlwaysClick(Sender: TObject);
begin
  fSecureMode:=3;
  end;

procedure TFtpDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult=mrOK then begin
    CanClose:=(length(hcbServer.Text)>0) or (length(hcbUser.Text)>0);
    if not Canclose then begin
      if length(hcbServer.Text)=0 then begin
        ErrorDialog('Es wurde kein Server angegeben!');
        hcbServer.SetFocus;
        end
      else begin
        ErrorDialog('Es wurde kein Benutzername angegeben!');
        hcbUser.SetFocus;
        end;
      end;
    end;
  end;

function TFtpDialog.Execute (var AFtpPar : TFtpParams; ShowPwd,ShowSecure : boolean;
                             ShowTitle : boolean) : boolean;
var
  i : integer;
begin
  sp:=ShowPwd;
  btnShow.Visible:=ShowPwd;
  with gbSecure do begin
    Enabled:=ShowSecure;
    for i:=0 to ControlCount-1 do Controls[i].Enabled:=ShowSecure;
    end;
  if ShowTitle then begin
    pnTitle.Show;
    ClientHeight:=pnSettings.Height+pnTitle.Height;
    end
  else begin
    ClientHeight:=pnSettings.Height;
    pnTitle.Hide;
    end;
  with AFtpPar do begin
    edName.Text:=Caption;
    hcbServer.Text:=Host;
    edtPort.Value:=Port;
    hcbUser.Text:=Username;
    edtFtpPwd.Text:=Password;
    hcbDir.Text:=Directory;
    if not ShowSecure then SecureMode:=0;  // secure mode not available
    FSecureMode:=SecureMode;
    case SecureMode of
    1 : rbAvail.Checked:=true;
    2 : rbRequired.Checked:=true;
    3 : rbAlways.Checked:=true;
    else rbNone.Checked:=true;
      end;
    cbPassive.Checked:=Passive;
    cbUseHost.Checked:=UseHost;
    cbFtpLog.Checked:=WriteLog;
    cbUtf8.Checked:=ForceUtf8;
    cbxCase.ItemIndex:=integer(CaseMode);
    FProxyPar:=Proxy;
    cbProxy.Checked:=FProxyPar.Mode<>fpcmNone;
    ShowProxy;
    if ShowModal=mrOK then begin
      Host:=hcbServer.Text;
      Port:=edtPort.Value;
      Username:=hcbUser.Text;
      Password:=edtFtpPwd.Text;
      Directory:=DosPathToUnixPath(hcbDir.Text);
      Caption:=edName.Text;
      if length(Caption)=0 then Caption:=GetFtpHint(AFtpPar);
      SecureMode:=FSecureMode;
      Passive:=cbPassive.Checked;
      UseHost:=cbUseHost.Checked;
      WriteLog:=cbFtpLog.Checked;
      ForceUtf8:=cbUtf8.Checked;
      CaseMode:=TTextChange(cbxCase.ItemIndex);
      if not cbProxy.Checked then FProxyPar.Mode:=fpcmNone;
      Proxy:=FProxyPar;
      Result:=true;
      end
    else Result:=false;
    end;
  end;

end.
