(* Delphi Dialog
   Anzeige eines Hilfefenster fr HTML-Hilfe
   =========================================
    
    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Sep. 2001 *)

unit HelpWin;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.OleCtrls,
  SHDocVw, WinUtils;

type
  THelpWindow = class(TForm)
    pnlTop: TPanel;
    BackBtn: TSpeedButton;
    FwdBtn: TSpeedButton;
    HomeBtn: TSpeedButton;
    EndeBtn: TSpeedButton;
    edURL: TEdit;
    btnSize1: TButton;
    btnSize2: TButton;
    btnSize3: TButton;
    procedure EndeBtnClick(Sender: TObject);
    procedure BackBtnClick(Sender: TObject);
    procedure FwdBtnClick(Sender: TObject);
    procedure HomeBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnSize1Click(Sender: TObject);
    procedure btnSize2Click(Sender: TObject);
    procedure btnSize3Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FIniName,FSection : string;
    WebBrowser : TWebBrowser;
    Available  : boolean;
    Home       : string;
    WSt        : TWindowState;
    HForm      : TRect;
    procedure NavigateComplete (Sender: TObject; const pDisp: IDispatch;
                                const URL: OleVariant);
  public
    { Public declarations }
  procedure LoadFromIni (AIniname,ASection : string);
  procedure Execute (Titel,URL : string);
  end;

var
  HelpWindow: THelpWindow;

implementation

{$R *.DFM}

uses IniFiles;

const
  IniLeft= 'Left';
  IniTop = 'Top';
  IniWidth = 'Width';
  IniHeight = 'Height';
  IniState = 'State';

procedure THelpWindow.LoadFromIni (AIniname,ASection : string);
var
  IniFile  : TIniFile;
begin
  FIniName:=AIniname; FSection:=ASection;
  IniFile:=TIniFile.Create(FIniName);
  with IniFile do begin
    with HForm do begin
      Left:=ReadInteger (FSection,IniLeft,10);
      Top:=ReadInteger (FSection,IniTop,10);
      Right:=ReadInteger (FSection,IniWidth,600);
      Bottom:=ReadInteger (FSection,IniHeight,450);
      end;
    WSt:=TWindowState(ReadInteger(FSection,IniState,Ord(wsNormal)));
    Free;
    end;
  Left:=HForm.Left; Top:=HForm.Top;
  Width:=HForm.Right; Height:=HForm.Bottom;
  WindowState:=WSt;
  end;

procedure THelpWindow.FormCreate(Sender: TObject);
begin
  Available:=true;
  FIniName:=''; FSection:='';
  end;

procedure THelpWindow.FormDestroy(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  if (length(FIniname)>0) and (length(FSection)>0) then begin
    IniFile:=TIniFile.Create(FIniName);
    with IniFile do begin
      if WindowState=wsNormal then begin
        WriteInteger (FSection,IniLeft,Left);
        WriteInteger (FSection,IniTop,Top);
        WriteInteger (FSection,IniWidth,Width);
        WriteInteger (FSection,IniHeight,Height);
        end;
      WriteInteger (FSection,IniState,ord(WindowState));
      Free;
      end;
    end;
  end;
procedure THelpWindow.NavigateComplete (Sender: TObject; const pDisp: IDispatch; 
                                        const URL: OleVariant);
begin
  edURL.Text:=WebBrowser.LocationURL;
  end;

{ ------------------------------------------------------------------- }
procedure THelpWindow.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

procedure THelpWindow.BackBtnClick(Sender: TObject);
begin
  try
    with WebBrowser do begin
      GoBack;
      end;
  except
    end;
  end;

procedure THelpWindow.FwdBtnClick(Sender: TObject);
begin
  try
    with WebBrowser do begin
      GoForward;
      end;
  except
    end;
  end;

procedure THelpWindow.HomeBtnClick(Sender: TObject);
begin
  with WebBrowser do begin
    Navigate(Home);
    end;
  end;

{ ------------------------------------------------------------------- }
procedure THelpWindow.Execute (Titel,URL      : string);
begin
  FreeAndNil(WebBrowser);
  try
    WebBrowser:=TWebBrowser.Create(self);
    with WebBrowser do begin
      ManualDock (self,nil,alNone);
      Align:=alClient;
      RegisterAsDropTarget:=false;
      OnNavigateComplete2:=NavigateComplete;
      end;
  except
    Available:=false;
    end;
  if Available then begin
    Home:=URL; Caption:=Titel;
    edURL.Text:=URL;
    WebBrowser.Navigate (URL);
    Show;
    end
  else ErrorDialog (Caption,'Die Online-Hilfe ist auf diesem System nicht verfgbar!'+sLineBreak+
                  'Installieren Sie bitte den Internet Explorer Vers. 4 oder neuer');
            //  'Die Online-Hilfe ist auf diesem System nicht verfgbar.'+sLineBreak+
            //  'Installieren Sie dazu den Internet Explorer Vers. 4 oder neuer.';
  end;

procedure THelpWindow.btnSize1Click(Sender: TObject);
begin
  ClientWidth:=640; ClientHeight:=480+pnlTop.Height;
  end;

procedure THelpWindow.btnSize2Click(Sender: TObject);
begin
  ClientWidth:=800; ClientHeight:=600+pnlTop.Height;
  end;

procedure THelpWindow.btnSize3Click(Sender: TObject);
begin
  ClientWidth:=1024; ClientHeight:=768+pnlTop.Height;
  end;

procedure THelpWindow.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_ESCAPE then Close;
  end;

end.
