(* Delphi Dialog
   Hinweisfenster fr Status mit Fortschrittsanzeige (opt.)
   ========================================================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Okt. 2002 
   *)

unit ProgWind;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls,
  Winapi.Messages;

const
  Rand = 5;
  Breite = 271;

  wmCancelAction = WM_USER+1;

type
  TProgressWindow = class(TForm)
    StatusLabel: TLabel;
    ProgressBar: TProgressBar;
    Button: TBitBtn;
    ActionLabel: TLabel;
    procedure ButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FHandle : HWND; // Handle of calling form
    procedure WritePosition (Value : integer);
    procedure SetStatus (AStat : string);
    procedure SetError (AStat : string);
    procedure SetAction (AAction : string);
  public
    { Public declarations }
    WStatus,
    Stopped : boolean;
    property Position : integer write WritePosition;
    property Action : string write SetAction;
    property Error : string write SetError;
    property Status : string write SetStatus;

    procedure ShowWindow (CfHandle : HWND; ATitle,AAction : string; AWidth,AMin,AMax : integer);
    procedure CloseWindow;
  end;

var
  ProgressWindow: TProgressWindow;

implementation

{$R *.DFM}
{$R *.RES}

uses FileCtrl;

procedure TProgressWindow.FormCreate(Sender: TObject);
begin
  FHandle:=0;
  Stopped:=false; WStatus:=false;
  end;

procedure TProgressWindow.FormResize(Sender: TObject);
var
  w : integer;
begin
  w:=Width;
  with Button do Left:=(w-Width) div 2;
  end;

{ ---------------------------------------------------------------- }
procedure TProgressWindow.WritePosition (Value : integer);
begin
  with ProgressBar do begin
    if Value<Min then Value:=Min;
    if Value>Max then Value:=Max;
    Position:=Value;
    end;
  end;

procedure TProgressWindow.SetStatus (AStat : string);
begin
  with StatusLabel do begin
    Font.Color:=clBlue;
    Caption:=MinimizeName(AStat,Canvas,ProgressBar.Width);
    end;
  end;

procedure TProgressWindow.SetError(AStat : string);
begin
  with StatusLabel do begin
    Font.Color:=clRed;
    Caption:=AStat;
    end;
  end;

procedure TProgressWindow.SetAction (AAction: string);
begin
  with ActionLabel do begin
    Caption:=AAction;
    end;
  end;

procedure TProgressWindow.ButtonClick(Sender: TObject);
begin
  Stopped:=true;
  if WStatus then Close
  else if FHandle<>0 then PostMessage (FHandle,wmCancelAction,0,0);
  end;

{ ---------------------------------------------------------------- }
(* Fortschrittsbalken anzeigen *)
procedure TProgressWindow.ShowWindow (CfHandle : HWND;
                                      ATitle,AAction   : string;
                                      AWidth,AMin,AMax : integer);
(* ATitle  : Fensterberschrift
   AAction : Name der Aktion
   AWidth  : Fensterbreite;
   Amin,Amax  : Randwerte des Balkens *)
var
  w : integer;
begin
  Stopped:=false; WStatus:=false;
  FHandle:=CfHandle;
  if AWidth>Width then Width:=AWidth;
  w:=ClientWidth;
  Caption:=ATitle;
  Action:=MinimizeName(AAction,Canvas,ClientWidth-2*ActionLabel.Left);
  Status:='';
  with Button do Left:=(w-Width) div 2;
  with ProgressBar do begin
    Width:=w-10;
    Min:=AMin; Max:=AMax;
    Position:=0;
    end;
  with Button do begin
    Glyph.LoadFromResourceName(HInstance,'CANCEL');
    Left:=(w-Width) div 2;
    Caption:='Abbrechen';
    end;
  Show;
  end;

procedure TProgressWindow.CloseWindow;
begin
  if Visible then Hide;
  if stopped then Action:='Abbruch durch den Benutzer!';
  WStatus:=true;
  with Button do begin
    Glyph.LoadFromResourceName(HInstance,'OK');
    Caption:='OK';
    end;
  messagebeep (mb_OK);
  Application.BringToFront;
  ShowModal;
  end;

end.
